"""
The :py:class:`PsmDataset` class is used to define a collection of peptide-spectrum matches
"""


class PsmDataset:
    """
    Store a collection of PSMs and their features.

    Parameters
        ----------
        data : pandas.DataFrame
            dataframe of PSMs with columns indicating spectrum, score, and target
        spectrum_col : str
            name of the column that identifies the psm
        score_col : str
            name of the column that defines the scores (p-values) of the psms
        target_col : str
            name of the column that indicates if a psm is a target/decoy


    Attributes
        ----------
        data : pandas.DataFrame
    """

    def __init__(self, data, spectrum_col, score_col, target_col):
        """
        Initialize a PsmDataset object.
        """

        self._data = data
        self.spectrum_col = spectrum_col
        self.score_col = score_col
        self.target_col = target_col

    @property
    def data(self):
        """The full data collection of PSMs as a :py:class:`pandas.DataFrame`."""
        return self._data

    def get_col(self, col_name):
        """The column specified by the col_name as a  :py:class:`pandas.DataFrame`."""
        return self._data.loc[:, col_name]



"""
The :py:class:`Result` class is used to define a collection of peptide-spectrum matches with calculated
False Discovery Rates and Q-Values.
"""

import os


class Result:
    """
    Store a collection of PSMs, their features, and their FDR/Q-Values

    Parameters
        ----------
        data : pandas.DataFrame
            dataframe of PSMs with columns indicating spectrum, score, and target
        spectrum_col : str or tuple of str
            one or more column names that identifies the psm
        score_col : str
            name of the column that defines the scores (p-values) of the psms
        target_col : str
            name of the column that indicates if a psm is a target/decoy


    Attributes
        ----------
        data : pandas.DataFrame
    """

    def __init__(self, data, spectrum_col, score_col, target_col):
        """
        Initialize a PsmDataset object.
        """

        self._data = data.reset_index(drop=True)
        self.spectrum_col = spectrum_col
        self.score_col = score_col
        self.target_col = target_col
        self.fdr_col = "FDR"
        self.q_val_col = "Q_Value"

    @property
    def data(self):
        """The full data collection of PSMs as a :py:class:`pandas.DataFrame`."""
        return self._data

    def get_col(self, col_name):
        """The column specified by the col_name as a  :py:class:`pandas.DataFrame`."""
        return self._data.loc[:, col_name]

    def write_file(self, output_dir=None, file_root=None):
        """
        Exports the data as a .txt file with the suffix "crema.psm_results.txt".

        Parameters
        ----------
        output_dir : str, optional
            The directory in which to save the files. Defaults to the current working directory if not specified.
        file_root : str, optional
            A prefix concatenated to the output result file. Defaults to none.

        Returns
        -------
        str
            The file path to the exported results file
        """
        out_file = "crema.psm_results.txt"
        if output_dir is None:
            output_dir = os.getcwd()
        if file_root is not None:
            out_file = file_root + out_file
        file_path = os.path.join(output_dir, out_file)
        self.data.to_csv(file_path)
        return file_path



"""
These tests verify that the crema CLI works as expected.
"""

import os
import subprocess


def test_cli_basic(tmp_path):
    """
    Test that the basic cli works. Reads in a file
    with crux default column names. Does nothing else.

    Parameters
    ----------
    tmp_path : pytest fixture of a temporary directory
        A pytest temporary directory unique to the test invocation

    Returns
    -------
    Pandas Assert Frame
        Asserts whether or not the the results file and log file
        are created properly with the correct file path.
    """
    cmd = ["crema", "data/single_basic.csv", "--output_dir", tmp_path]
    subprocess.run(cmd, check=True)
    assert os.path.isfile(os.path.join(tmp_path, "crema.psm_results.txt"))
    assert os.path.isfile(os.path.join(tmp_path, "crema.logfile.log"))


def test_cli_basic_tab(tmp_path):
    """
    Test that the cli works with a tab delimited file. Reads in a file
    with crux default column names.

    Parameters
    ----------
    tmp_path : pytest fixture of a temporary directory
        A pytest temporary directory unique to the test invocation

    Returns
    -------
    Pandas Assert Frame
        Asserts whether or not the the results file and log file
        are created properly with the correct file path.
    """
    cmd = ["crema", "data/single_basic_tab.txt", "--output_dir", tmp_path]
    subprocess.run(cmd, check=True)
    assert os.path.isfile(os.path.join(tmp_path, "crema.psm_results.txt"))
    assert os.path.isfile(os.path.join(tmp_path, "crema.logfile.log"))


def test_cli_custom_root(tmp_path):
    """
    Test that the cli works with custom file root.
    Reads in a file with crux default column names.
    Specifies a custom file_root string to tag on
    as a prefix to the output files.

    Parameters
    ----------
    tmp_path : pytest fixture of a temporary directory
        A pytest temporary directory unique to the test invocation

    Returns
    -------
    Pandas Assert Frame
        Asserts whether or not the the results file and log file
        are created properly with the correct file path.
    """
    cmd = [
        "crema",
        "data/single_basic.csv",
        "--output_dir",
        tmp_path,
        "--file_root",
        "myFileRoot",
    ]
    subprocess.run(cmd, check=True)
    assert os.path.isfile(
        os.path.join(tmp_path, "myFileRootcrema.psm_results.txt")
    )
    assert os.path.isfile(
        os.path.join(tmp_path, "myFileRootcrema.logfile.log")
    )


def test_cli_custom_param(tmp_path):
    """
    Test that the cli works when specifying non-default column
    names. Reads in a file with non-crux default column names.
    Specifies different score and target columns to read.

    Parameters
    ----------
    tmp_path : pytest fixture of a temporary directory
        A pytest temporary directory unique to the test invocation

    Returns
    -------
    Pandas Assert Frame
        Asserts whether or not the the results file and log file
        are created properly with the correct file path.
    """
    cmd = [
        "crema",
        "data/single_noncrux.csv",
        "--output_dir",
        tmp_path,
        "--score",
        "p-value",
        "--target",
        "target",
    ]
    subprocess.run(cmd, check=True)
    assert os.path.isfile(os.path.join(tmp_path, "crema.psm_results.txt"))
    assert os.path.isfile(os.path.join(tmp_path, "crema.logfile.log"))


def test_cli_int_targets(tmp_path):
    """
    Test that the cli works with integer value target
    column values. Reads in a file
    with crux default column names. Does nothing else.

    Parameters
    ----------
    tmp_path : pytest fixture of a temporary directory
        A pytest temporary directory unique to the test invocation

    Returns
    -------
    Pandas Assert Frame
        Asserts whether or not the the results file and log file
        are created properly with the correct file path.
    """
    cmd = [
        "crema",
        "data/single_int_targets.csv",
        "--output_dir",
        tmp_path,
    ]
    subprocess.run(cmd, check=True)
    assert os.path.isfile(os.path.join(tmp_path, "crema.psm_results.txt"))
    assert os.path.isfile(os.path.join(tmp_path, "crema.logfile.log"))


def test_cli_text_scan(tmp_path):
    """
    Test that the cli works with String value scan
    column values. Reads in a file
    with crux default column names. Does nothing else.

    Parameters
    ----------
    tmp_path : pytest fixture of a temporary directory
        A pytest temporary directory unique to the test invocation

    Returns
    -------
    Pandas Assert Frame
        Asserts whether or not the the results file and log file
        are created properly with the correct file path.
    """
    cmd = [
        "crema",
        "data/single_text_scan.csv",
        "--output_dir",
        tmp_path,
    ]
    subprocess.run(cmd, check=True)
    assert os.path.isfile(os.path.join(tmp_path, "crema.psm_results.txt"))
    assert os.path.isfile(os.path.join(tmp_path, "crema.logfile.log"))


def test_cli_add_spectrum(tmp_path):
    """
    Test that the cli works with more than one spectrum
    column. Reads in a file with crux default column names.
    Specifies two scan columns to read in.

    Parameters
    ----------
    tmp_path : pytest fixture of a temporary directory
        A pytest temporary directory unique to the test invocation

    Returns
    -------
    Pandas Assert Frame
        Asserts whether or not the the results file and log file
        are created properly with the correct file path.
    """
    cmd = [
        "crema",
        "data/single_add_spectrum.csv",
        "--spectrum",
        "scan",
        "extras",
        "--output_dir",
        tmp_path,
    ]
    subprocess.run(cmd, check=True)
    assert os.path.isfile(os.path.join(tmp_path, "crema.psm_results.txt"))
    assert os.path.isfile(os.path.join(tmp_path, "crema.logfile.log"))


def test_cli_multi(tmp_path):
    """
    Test that the cli works with multiple
    file inputs. Reads in two files
    with crux default column names. Does nothing else.

    Parameters
    ----------
    tmp_path : pytest fixture of a temporary directory
        A pytest temporary directory unique to the test invocation

    Returns
    -------
    Pandas Assert Frame
        Asserts whether or not the the results file and log file
        are created properly with the correct file path.
    """
    cmd = [
        "crema",
        "data/multi_target.csv",
        "data/multi_decoy.csv",
        "--output_dir",
        tmp_path,
    ]
    subprocess.run(cmd, check=True)
    assert os.path.isfile(os.path.join(tmp_path, "crema.psm_results.txt"))
    assert os.path.isfile(os.path.join(tmp_path, "crema.logfile.log"))



"""
This is the command line interface for crema
"""
import os
import sys
import time
import logging

from .parsers import *
from .params import Params
from .methods import calculate_tdc


def main():
    """The CLI entry point"""
    start_time = time.time()

    # Creates the parser for parse args and reads in command line arguments
    params = Params().parser
    args = params.parse_args()

    # Set up logging files
    log_file = "crema.logfile.log"
    if args.file_root is not None:
        log_file = args.file_root + log_file
    if args.output_dir is None:
        args.output_dir = os.getcwd()

    # Configure logging
    logging.basicConfig(
        filename=os.path.join(args.output_dir, log_file),
        level=logging.INFO,
        format="%(asctime)s %(message)s",
    )

    logging.info("crema")
    logging.info("Written by Donavan See (seed99@cs.washington.edu) in the")
    logging.info(
        "Department of Genome Sciences at the University of " "Washington."
    )
    logging.info("Command issued:")
    logging.info("%s", " ".join(sys.argv))
    logging.info("")
    logging.info("Starting Analysis")
    logging.info("=================")

    # Create dataset object
    logging.info("Creating dataset object...")
    psms = read_file(args.input_files, args.spectrum, args.score, args.target)

    # Run confidence estimate method
    logging.info("Calculating confidence estimate...")
    result = calculate_tdc(psms)

    # Write result to file
    logging.info("Writing to file...")
    result.write_file(output_dir=args.output_dir, file_root=args.file_root)

    # Calculate how long the confidence estimation took
    end_time = time.time()
    total_time = end_time - start_time

    logging.info("=== DONE! ===")
    logging.info("Time Taken:" + str(total_time))


if __name__ == "__main__":
    main()
